@@ -4,6 +4,7 @@
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
+#include <QApplication>
 #include <QCommandLineParser>
 #include <QIcon>
 #include <QQmlApplicationEngine>
@@ -14,10 +15,10 @@
 
 #include <KLocalizedString>
 
+#include <KDSingleApplication>
+
 #include "application.h"
 #include "generalsettings.h"
-#include "kdsingleapplication.h"
-#include "qqmlpropertymap.h"
 #include "thumbnailimageprovider.h"
 
 using namespace Qt::StringLiterals;
@@ -79,7 +80,6 @@ int main(int argc, char *argv[])
 
     QQmlApplicationEngine engine(&qApplication);
     engine.addImageProvider(u"thumbnail"_s, new ThumbnailImageProvider());
-    engine.rootContext()->setContextProperty(u"appActions"_s, new QQmlPropertyMap);
     engine.rootContext()->setContextObject(new KLocalizedContext(Application::instance()));
     engine.loadFromModule("org.kde.haruna", "Main");
 
