@@ -8,7 +8,6 @@
 #define TRACKSMODEL_H
 
 #include <QAbstractListModel>
-#include <QObject>
 #include <QtQml/qqmlregistration.h>
 
 struct Track {
@@ -50,7 +49,7 @@ public Q_SLOTS:
     void setTracks(QList<Track> tracks);
 
 private:
-    QList<Track> m_tracks;
+    QList<Track> m_data;
 };
 
 #endif // TRACKSMODEL_H
