@@ -1,11 +1,13 @@
 /*
  * SPDX-FileCopyrightText: 2020 George Florea Bănuș <georgefb899@gmail.com>
+ * SPDX-FileCopyrightText: 2025 Muhammet Sadık Uğursoy <sadikugursoy@gmail.com>
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #include "mpvitem.h"
 
+#include <QApplication>
 #include <QCommandLineParser>
 #include <QCryptographicHash>
 #include <QDir>
@@ -21,42 +23,41 @@
 #include "audiosettings.h"
 #include "chaptersmodel.h"
 #include "config-haruna.h"
+#include "database.h"
 #include "generalsettings.h"
 #include "global.h"
+#include "lockmanager.h"
 #include "mpvproperties.h"
 #include "playbacksettings.h"
+#include "playlistfilterproxymodel.h"
 #include "playlistmodel.h"
+#include "playlistmultiproxiesmodel.h"
 #include "playlistsettings.h"
+#include "recentfilesmodel.h"
 #include "subtitlessettings.h"
 #include "tracksmodel.h"
 #include "videosettings.h"
 #include "worker.h"
+#include "youtube.h"
 
 #if defined(Q_OS_UNIX)
 #include <QDBusConnection>
 
-#include "lockmanager.h"
 #include "mediaplayer2.h"
 #include "mediaplayer2player.h"
 #endif
 
-#if defined(Q_OS_WIN32)
-#include <Windows.h>
-#endif
-
 using namespace Qt::StringLiterals;
 
 MpvItem::MpvItem(QQuickItem *parent)
     : MpvAbstractItem(parent)
     , m_audioTracksModel{std::make_unique<TracksModel>()}
     , m_subtitleTracksModel{std::make_unique<TracksModel>()}
-    , m_playlistModel{std::make_unique<PlaylistModel>()}
-    , m_playlistProxyModel{std::make_unique<PlaylistProxyModel>()}
+    , m_playlists{std::make_unique<PlaylistMultiProxiesModel>()}
     , m_chaptersModel{std::make_unique<ChaptersModel>()}
     , m_watchLaterPath{QString(Global::instance()->appConfigDirPath()).append(u"/watch-later/"_s)}
     , m_saveTimePositionTimer{std::make_unique<QTimer>()}
 {
-    m_playlistProxyModel->setSourceModel(m_playlistModel.get());
     Q_EMIT observeProperty(MpvProperties::self()->MediaTitle, MPV_FORMAT_STRING);
     Q_EMIT observeProperty(MpvProperties::self()->Position, MPV_FORMAT_DOUBLE);
     Q_EMIT observeProperty(MpvProperties::self()->Remaining, MPV_FORMAT_DOUBLE);
@@ -74,6 +75,7 @@ MpvItem::MpvItem(QQuickItem *parent)
     Q_EMIT observeProperty(MpvProperties::self()->ChapterList, MPV_FORMAT_NODE);
     Q_EMIT observeProperty(MpvProperties::self()->TracksCount, MPV_FORMAT_NODE);
     Q_EMIT observeProperty(MpvProperties::self()->SubtitleDelay, MPV_FORMAT_DOUBLE);
+    Q_EMIT observeProperty(MpvProperties::self()->EofReached, MPV_FORMAT_FLAG);
 
     setupConnections();
     initProperties();
@@ -133,7 +135,8 @@ void MpvItem::initProperties()
     Q_EMIT setProperty(MpvProperties::self()->VolumeMax, 100);
 
     // set ytdl_path to yt-dlp or fallback to youtube-dl
-    Q_EMIT setProperty(MpvProperties::self()->ScriptOpts, u"ytdl_hook-ytdl_path=%1"_s.arg(Application::youtubeDlExecutable()));
+    YouTube yt;
+    Q_EMIT setProperty(MpvProperties::self()->ScriptOpts, u"ytdl_hook-ytdl_path=%1"_s.arg(yt.youtubeDlExecutable()));
     QCommandLineParser *cmdParser = Application::instance()->parser();
     QString ytdlFormat = PlaybackSettings::ytdlFormat();
     if (cmdParser->isSet(u"ytdl-format-selection"_s)) {
@@ -153,7 +156,30 @@ void MpvItem::initProperties()
     Q_EMIT setProperty(MpvProperties::self()->SubtitleBorderSize, SubtitlesSettings::borderSize());
     Q_EMIT setProperty(MpvProperties::self()->SubtitleBold, SubtitlesSettings::isBold());
     Q_EMIT setProperty(MpvProperties::self()->SubtitleItalic, SubtitlesSettings::isItalic());
-    Q_EMIT setProperty(MpvProperties::self()->SubtitleFilePaths, SubtitlesSettings::subtitlesFolders().join(u":"_s));
+
+    auto setSubPaths = [this]() {
+        QString subFoldersString;
+        const auto subFolders{SubtitlesSettings::subtitlesFolders()};
+
+        for (const auto &sf : subFolders) {
+            if (sf.startsWith(u"*"_s)) {
+                if (!SubtitlesSettings::recursiveSubtitlesSearch()) {
+                    // only add relative folders starting with * if recursive search is disabled
+                    // otherwise both recursive search and mpv will find and add the same sub file
+                    QString _sf{sf};
+                    _sf = _sf.removeFirst();
+                    subFoldersString.append(u"%1:"_s.arg(_sf));
+                }
+            } else {
+                subFoldersString.append(u"%1:"_s.arg(sf));
+            }
+        }
+        subFoldersString.removeLast();
+
+        Q_EMIT setProperty(MpvProperties::self()->SubtitleFilePaths, subFoldersString);
+    };
+    setSubPaths();
+    connect(SubtitlesSettings::self(), &SubtitlesSettings::RecursiveSubtitlesSearchChanged, this, setSubPaths);
     selectSubtitleTrack();
 
     Q_EMIT setProperty(MpvProperties::self()->ScreenshotTemplate, VideoSettings::screenshotTemplate());
@@ -177,6 +203,9 @@ void MpvItem::setupConnections()
     connect(mpvController(), &MpvController::endFile,
             this, &MpvItem::onEndFile, Qt::QueuedConnection);
 
+    connect(this, &MpvItem::eofReachedChanged,
+            this, &MpvItem::onEndOfFileReached);
+
     connect(mpvController(), &MpvController::videoReconfig,
             this, &MpvItem::videoReconfig, Qt::QueuedConnection);
 
@@ -211,18 +240,29 @@ void MpvItem::setupConnections()
 
     connect(Worker::instance(), &Worker::subtitlesFound, this, [this](QStringList subs) {
         for (const auto &sub : std::as_const(subs)) {
-            command({u"sub-add"_s, sub, u"select"_s});
-            loadTracks(getProperty(MpvProperties::self()->TrackList).toList());
+            commandBlocking(QStringList{u"sub-add"_s, sub, u"select"_s});
         }
+        getPropertyAsync(MpvProperties::self()->TrackList, static_cast<int>(AsyncIds::TrackList));
     });
 
     connect(this, &MpvItem::chapterChanged,
             this, &MpvItem::onChapterChanged);
 
-    connect(m_playlistModel.get(), &PlaylistModel::playingItemChanged, this, [=]() {
-        loadFile(m_playlistModel->m_playlist[m_playlistModel->m_playingItem].url.toString());
+    connect(m_playlists.get(), &PlaylistMultiProxiesModel::playingItemChanged, this, [=]() {
+        const auto playlistModel = activeFilterProxyModel()->playlistModel();
+        const auto url = playlistModel->m_playlist[playlistModel->m_playingItem].url;
+        const auto mediaTitle = playlistModel->m_playlist[playlistModel->m_playingItem].mediaTitle;
+        loadFile(url.toString());
+        Q_EMIT addToRecentFiles(url, RecentFilesModel::OpenedFrom::Playlist, mediaTitle);
     });
 
+    connect(m_playlists.get(), &PlaylistMultiProxiesModel::visibleIndexChanged, this, [=]() {
+        Q_EMIT visibleFilterProxyModelChanged();
+    });
+
+    connect(m_playlists.get(), &PlaylistMultiProxiesModel::activeIndexChanged, this, [=]() {
+        Q_EMIT activeFilterProxyModelChanged();
+    });
 
 #if HAVE_DBUS
     // register mpris dbus service
@@ -260,47 +300,47 @@ void MpvItem::setupConnections()
     });
 #endif
 
-#if defined(Q_OS_UNIX)
     connect(this, &MpvItem::pauseChanged, this, [=]() {
         static LockManager lockManager;
-        if (!pause() && !m_currentUrl.isEmpty()) {
-            lockManager.setInhibitionOn();
-        } else {
+        if (pause()) {
             lockManager.setInhibitionOff();
+        } else {
+            if (!m_currentUrl.isEmpty()) {
+                lockManager.setInhibitionOn();
+            }
+            const auto order = PlaylistSettings::playbackBehavior();
+            if (order != u"StopAfterLast"_s && order != u"StopAfterItem"_s) {
+            }
         }
     });
-#endif
 
-#if defined(Q_OS_WIN32)
-connect(this, &MpvItem::pauseChanged, this, [=]() {
-    if (!pause() && !m_currentUrl.isEmpty()) {
-        SetThreadExecutionState(ES_CONTINUOUS | ES_DISPLAY_REQUIRED);
-    } else {
-        SetThreadExecutionState(ES_CONTINUOUS);
-    }
-});
-#endif
-
-    connect(this, &MpvItem::syncConfigValue, Worker::instance(), &Worker::syncConfigValue, Qt::QueuedConnection);
+    connect(this, &MpvItem::savePositionToDB, Worker::instance(), &Worker::savePositionToDB, Qt::QueuedConnection);
     // clang-format on
 }
 
 void MpvItem::onReady()
 {
     setIsReady(true);
-
+    auto proxyModel = m_playlists->defaultFilterProxy();
     QUrl url{Application::instance()->url(0)};
     if (!url.isEmpty() && url.isValid()) {
-        playlistModel()->addItem(Application::instance()->url(0), PlaylistModel::Clear);
-        Q_EMIT addToRecentFiles(url);
+        proxyModel->addItem(Application::instance()->url(0), PlaylistModel::Clear);
+        Q_EMIT addToRecentFiles(url, RecentFilesModel::OpenedFrom::ExternalApp, url.fileName());
     } else {
         if (PlaybackSettings::openLastPlayedFile()) {
-            // if both lastPlaylist and lastPlayedFile are set the playlist is loaded
-            // and the lastPlayedFile is searched in the playlist
-            if (!GeneralSettings::lastPlaylist().isEmpty()) {
-                playlistModel()->addItem(GeneralSettings::lastPlaylist(), PlaylistModel::Clear);
+            // Internal playlists are read at this point and active playlist is set. If an internal playlist
+            // is active, make it visible too and start playing.
+            if (m_playlists->m_activeIndex != 0) {
+                m_playlists->setVisibleIndex(m_playlists->m_activeIndex);
+                activeFilterProxyModel()->setPlayingItem(activeFilterProxyModel()->playlistModel()->m_playingItem);
             } else {
-                playlistModel()->addItem(GeneralSettings::lastPlayedFile(), PlaylistModel::Clear);
+                // if both lastPlaylist and lastPlayedFile are set the playlist is loaded
+                // and the lastPlayedFile is searched in the playlist
+                if (!GeneralSettings::lastPlaylist().isEmpty()) {
+                    proxyModel->addItem(GeneralSettings::lastPlaylist(), PlaylistModel::Clear);
+                } else {
+                    proxyModel->addItem(GeneralSettings::lastPlayedFile(), PlaylistModel::Clear);
+                }
             }
         }
     }
@@ -308,33 +348,53 @@ void MpvItem::onReady()
 
 void MpvItem::onEndFile(const QString &reason)
 {
-    int currentItem = playlistProxyModel()->getPlayingItem();
+    // this runs after the file has been unloaded from mpv
     if (reason == u"error"_s) {
-        if (playlistModel()->rowCount() == 0) {
+        auto proxyModel = activeFilterProxyModel();
+        if (proxyModel->rowCount() == 0) {
             return;
         }
 
-        const auto index = playlistProxyModel()->index(currentItem, 0);
-        const auto title = playlistModel()->data(playlistProxyModel()->mapToSource(index), PlaylistModel::TitleRole);
+        uint currentItem = proxyModel->getPlayingItem();
+        const auto index = proxyModel->index(currentItem, 0);
+        const auto title = proxyModel->data(index, PlaylistModel::TitleRole);
 
-        Q_EMIT osdMessage(i18nc("@info:tooltip", "Could not play: %1", title.toString()));
+        Q_EMIT Application::instance()->error(i18nc("@info:tooltip; %1 is a video title/filename", "Could not play: %1", title.toString()));
+        return;
     }
+}
 
-    if (!playlistProxyModel()->isLastItem(currentItem)) {
-        playlistProxyModel()->playNext();
+void MpvItem::onEndOfFileReached()
+{
+    // this runs before the file is unloaded from mpv
+    if (!m_eofReached) {
         return;
     }
 
-    // Last file in playlist
-    if (PlaylistSettings::repeat()) {
-        // repeat is on -> open first item in the playlist
-        playlistProxyModel()->setPlayingItem(0);
+    auto proxyModel = activeFilterProxyModel();
+    const auto behavior = PlaylistSettings::playbackBehavior();
+    if (behavior == u"StopAfterLast"_s) {
+        uint currentItem = proxyModel->getPlayingItem();
+        if (proxyModel->isLastItem(currentItem)) {
+            setPropertyBlocking(MpvProperties::self()->Position, 0);
+            setPropertyBlocking(MpvProperties::self()->Pause, true);
+            return;
+        }
+    }
+
+    if (behavior == u"RepeatItem"_s) {
+        setPropertyBlocking(MpvProperties::self()->Position, 0);
+        setPropertyBlocking(MpvProperties::self()->Pause, false);
         return;
     }
 
-    // repeat is off -> reopen the last file paused
-    setPropertyBlocking(MpvProperties::self()->Pause, true);
-    Q_EMIT command(QStringList() << u"loadfile"_s << m_currentUrl.toString());
+    if (behavior == u"StopAfterItem"_s) {
+        setPropertyBlocking(MpvProperties::self()->Position, 0);
+        setPropertyBlocking(MpvProperties::self()->Pause, true);
+        return;
+    }
+
+    proxyModel->playNext();
 }
 
 void MpvItem::onPropertyChanged(const QString &property, const QVariant &value)
@@ -412,17 +472,32 @@ void MpvItem::onPropertyChanged(const QString &property, const QVariant &value)
     } else if (property == MpvProperties::self()->Height) {
         m_videoHeight = value.toInt();
         Q_EMIT videoHeightChanged();
+
+    } else if (property == MpvProperties::self()->EofReached) {
+        m_eofReached = value.toBool();
+        Q_EMIT eofReachedChanged();
     }
 }
 
 void MpvItem::loadFile(const QString &file)
 {
+    // must be set to always for the playback behavior to work as intended
+    Q_EMIT setProperty(MpvProperties::self()->KeepOpen, u"always"_s);
+
     auto url = QUrl::fromUserInput(file);
     if (m_currentUrl != url) {
         m_currentUrl = url;
         Q_EMIT currentUrlChanged();
     }
 
+    QFileInfo fi{url.toString(QUrl::PreferLocalFile)};
+    if (url.isLocalFile() && !fi.exists()) {
+        m_currentUrl = {};
+        Q_EMIT currentUrlChanged();
+        Q_EMIT Application::instance()->error(i18nc("@info:tooltip; %1 is an absolute path", "File doesn't exist: %1", file));
+        return;
+    }
+
     // store the mute property so it can be restored after loading file
     auto mute = m_mute;
     // mute to avoid popping sound while loading files
@@ -485,8 +560,7 @@ void MpvItem::onAsyncReply(const QVariant &data, mpv_event event)
     }
     case AsyncIds::SavePosition: {
         auto hash = md5(currentUrl().toString());
-        auto watchLaterConfig = QString(m_watchLaterPath).append(hash);
-        Q_EMIT syncConfigValue(watchLaterConfig, QString(), u"TimePosition"_s, data.toString());
+        Q_EMIT savePositionToDB(hash, currentUrl().toString(), data.toDouble());
         break;
     }
     case AsyncIds::Screenshot: {
@@ -498,7 +572,7 @@ void MpvItem::onAsyncReply(const QVariant &data, mpv_event event)
         if (filename.isEmpty()) {
             Q_EMIT osdMessage(i18nc("@info:tooltip osd", "Screenshot taken"));
         } else {
-            Q_EMIT osdMessage(i18nc("@info:tooltip osd", "Screenshot: %1", filename));
+            Q_EMIT osdMessage(i18nc("@info:tooltip osd; %1 is a filename with path", "Screenshot: %1", filename));
         }
         break;
     }
@@ -567,11 +641,8 @@ void MpvItem::onChapterChanged()
 
 void MpvItem::saveTimePosition()
 {
-    if (!PlaybackSettings::restoreFilePosition()) {
-        return;
-    }
     // position is saved only for files longer than PlaybackSettings::minDurationToSavePosition()
-    if (getProperty(MpvProperties::self()->Duration).toInt() < PlaybackSettings::minDurationToSavePosition() * 60) {
+    if (duration() < PlaybackSettings::minDurationToSavePosition() * 60) {
         return;
     }
 
@@ -580,10 +651,8 @@ void MpvItem::saveTimePosition()
 
 double MpvItem::loadTimePosition()
 {
-    if (!PlaybackSettings::restoreFilePosition()) {
-        return 0;
-    }
-    PlaylistItem item{m_playlistModel->m_playlist[m_playlistModel->m_playingItem]};
+    const auto playlistModel = activeFilterProxyModel()->playlistModel();
+    PlaylistItem item{playlistModel->m_playlist[playlistModel->m_playingItem]};
     auto duration{item.duration};
 
     if (qFuzzyCompare(duration, 0.0)) {
@@ -606,11 +675,7 @@ double MpvItem::loadTimePosition()
     }
 
     auto hash = md5(currentUrl().toString());
-    auto watchLaterConfig = QString(m_watchLaterPath).append(hash);
-    KConfig config(watchLaterConfig);
-    auto pos = config.group(QString()).readEntry("TimePosition", QString::number(0)).toDouble();
-
-    return pos;
+    return Database::instance()->playbackPosition(hash);
 }
 
 void MpvItem::resetTimePosition()
@@ -648,24 +713,29 @@ QString MpvItem::md5(const QString &str)
     return QString::fromUtf8(md5.toHex());
 }
 
-PlaylistModel *MpvItem::playlistModel()
+PlaylistFilterProxyModel *MpvItem::activeFilterProxyModel()
 {
-    return m_playlistModel.get();
+    return m_playlists->activeFilterProxy();
 }
 
-void MpvItem::setPlaylistModel(PlaylistModel *model)
+PlaylistFilterProxyModel *MpvItem::visibleFilterProxyModel()
 {
-    m_playlistModel.reset(model);
+    return m_playlists->visibleFilterProxy();
 }
 
-PlaylistProxyModel *MpvItem::playlistProxyModel()
+PlaylistFilterProxyModel *MpvItem::defaultFilterProxyModel()
 {
-    return m_playlistProxyModel.get();
+    return m_playlists->defaultFilterProxy();
 }
 
-void MpvItem::setPlaylistProxyModel(PlaylistProxyModel *model)
+PlaylistMultiProxiesModel *MpvItem::playlists()
 {
-    m_playlistProxyModel.reset(model);
+    return m_playlists.get();
+}
+
+void MpvItem::setPlaylists(PlaylistMultiProxiesModel *model)
+{
+    m_playlists.reset(model);
 }
 
 ChaptersModel *MpvItem::chaptersModel() const
