@@ -1,5 +1,6 @@
 ﻿/*
  * SPDX-FileCopyrightText: 2020 George Florea Bănuș <georgefb899@gmail.com>
+ * SPDX-FileCopyrightText: 2025 Muhammet Sadık Uğursoy <sadikugursoy@gmail.com>
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
@@ -11,9 +12,11 @@
 
 #include <memory>
 
+#include "recentfilesmodel.h"
+
 class ChaptersModel;
-class PlaylistModel;
-class PlaylistProxyModel;
+class PlaylistFilterProxyModel;
+class PlaylistMultiProxiesModel;
 class TracksModel;
 class MpvRenderer;
 
@@ -37,13 +40,18 @@ public:
     };
     Q_ENUM(AsyncIds)
 
-    Q_PROPERTY(PlaylistModel *playlistModel READ playlistModel WRITE setPlaylistModel NOTIFY playlistModelChanged)
-    PlaylistModel *playlistModel();
-    void setPlaylistModel(PlaylistModel *model);
+    Q_PROPERTY(PlaylistFilterProxyModel *activeFilterProxyModel READ activeFilterProxyModel NOTIFY activeFilterProxyModelChanged)
+    PlaylistFilterProxyModel *activeFilterProxyModel();
 
-    Q_PROPERTY(PlaylistProxyModel *playlistProxyModel READ playlistProxyModel WRITE setPlaylistProxyModel NOTIFY playlistProxyModelChanged)
-    PlaylistProxyModel *playlistProxyModel();
-    void setPlaylistProxyModel(PlaylistProxyModel *model);
+    Q_PROPERTY(PlaylistFilterProxyModel *visibleFilterProxyModel READ visibleFilterProxyModel NOTIFY visibleFilterProxyModelChanged)
+    PlaylistFilterProxyModel *visibleFilterProxyModel();
+
+    Q_PROPERTY(PlaylistFilterProxyModel *defaultFilterProxyModel READ defaultFilterProxyModel NOTIFY defaultFilterProxyModelChanged)
+    PlaylistFilterProxyModel *defaultFilterProxyModel();
+
+    Q_PROPERTY(PlaylistMultiProxiesModel *playlists READ playlists WRITE setPlaylists NOTIFY playlistsChanged)
+    PlaylistMultiProxiesModel *playlists();
+    void setPlaylists(PlaylistMultiProxiesModel *model);
 
     Q_PROPERTY(TracksModel *audioTracksModel READ audioTracksModel NOTIFY audioTracksModelChanged)
     TracksModel *audioTracksModel() const;
@@ -149,8 +157,10 @@ public:
 Q_SIGNALS:
     void audioTracksModelChanged();
     void subtitleTracksModelChanged();
-    void playlistModelChanged();
-    void playlistProxyModelChanged();
+    void activeFilterProxyModelChanged();
+    void visibleFilterProxyModelChanged();
+    void defaultFilterProxyModelChanged();
+    void playlistsChanged();
     void chaptersModelChanged();
     void finishedLoadingChanged();
     void playlistTitleChanged();
@@ -177,10 +187,11 @@ Q_SIGNALS:
     void videoReconfig();
     void videoWidthChanged();
     void videoHeightChanged();
-    void syncConfigValue(QString path, QString group, QString key, QVariant value);
+    void eofReachedChanged();
+    void savePositionToDB(const QString &md5Hash, const QString &path, double position);
 
     void osdMessage(const QString &text);
-    void addToRecentFiles(const QUrl &url);
+    void addToRecentFiles(const QUrl &url, RecentFilesModel::OpenedFrom from, const QString &name);
 
     // signals used for mpris
     void raise();
@@ -193,6 +204,7 @@ private:
     void setupConnections();
     void onReady();
     void onEndFile(const QString &reason);
+    void onEndOfFileReached();
     void onPropertyChanged(const QString &property, const QVariant &value);
     void saveTimePosition();
     double loadTimePosition();
@@ -206,8 +218,7 @@ private:
     std::unique_ptr<TracksModel> m_subtitleTracksModel;
     QList<int> m_secondsWatched;
     double m_watchPercentage{0.0};
-    std::unique_ptr<PlaylistModel> m_playlistModel;
-    std::unique_ptr<PlaylistProxyModel> m_playlistProxyModel;
+    std::unique_ptr<PlaylistMultiProxiesModel> m_playlists;
 
     double m_position{0.0};
     QString m_formattedPosition;
@@ -226,6 +237,7 @@ private:
     int m_secondarySubtitleId{0};
     int m_videoWidth{0};
     int m_videoHeight{0};
+    bool m_eofReached{false};
     QList<QVariant> m_chaptersList;
 
     double m_watchLaterPosition{0.0};
